#!/bin/env python
#-*- coding: utf-8 -*-
# 1行目は cygwin で使うときのおまじない
# 2行目は日本語コードの指定
#
# LucasLehmer.py
# Lucas-Lehmer アルゴリズムによる偶数の完全数の探索
# 2007.5.21
#
# Mersenne 数 2^n-1 が素数であるとき
#              2^(n-1) * (2^n-1) 
# は完全数であり、かつ偶数の完全数はこの形に限る。

from math import *	# 数学関数を使うためのまじない
from time import *	# 時間関数を使うためのまじない
from sys import *	# システム関数を使うためのまじない

# 素朴な素数判定法（n が小さい場合）
# n が素数なら 1, そうでなければ 0 を返す
def primetest(n):
	if n <= 1:
		return 0
	if n == 2:
		return 1
	if n % 2 == 0:
		return 0
	h = 1
	m = sqrt(n)
	i = 3L	
	while i <= m and h == 1:
		if n % i == 0:
			h = 0
		else:
			i = i + 2
	if h == 1:
		return 1
	else:
		return 0

# Mersenne 数 2^n -1 の素数判定
# 素数なら 1、そうでなければ 0 を返す
# n >= 3 のとき有効
def mersenne_primality(n):
	m = (2 ** n) - 1
	s = 4
	for i in range(2,n):
		s = (s * s - 2) % m
	if s == 0:
		return 1
	else:
		return 0

print u"偶数の完全数を探索します。"
print u"真の約数の和が自分自身になる数を完全数と言います。"
print u"何番目まで探しますか。",
last = input()
print

print u"1 個目の偶数の完全数は"
print u"2^1 × (2^2 - 1) = 6"
print

t0 = time()		# 開始時刻
n = 3
c = 1
while c < last:
	if primetest(n):
		if mersenne_primality(n):
			c += 1
			a = ( 2 ** (n-1) ) * ( 2 ** n - 1 )
			print u"%d 個目の偶数の完全数は" % c
			print u"2^%d × (2^%d - 1) = %d" % (n-1,n,a)
			t1 = time() - t0
			print u"（%f 秒経過）" % t1
			print
	n += 2

