#!/bin/env python
#-*- coding: utf-8 -*-
# 1行目は cygwin で使うときのおまじない
# 2行目は日本語コードの指定
#
# algeq.py
# 代数方程式の解（ニュートン法）
# 2007.5.22-23

from math import *
from cmath import *
from random import *
from sys import *

# 多項式は、次数と係数リストによる二重リストで表現する。
# 係数リストは降順で格納し、
# 例えば [3,[1,2,0,3]] は x^3 + 2 x^2 + 3 を表すこととする。

# 多項式の微分を返す
def polder(f):
        if f[0] == 0:
                g = [0,[0]]
        else:
                d = f[0]
                c = []
                for i in range(0,d):
                        c.append((d-i)*f[1][i])
                g = [d-1,c]
        return g

# 多項式 f(x) の x = a での値を返す
def poleval(f,a):
        w = 0
        d = f[0]
        for i in range(0,d+1):
                w = a * w + f[1][i]
        return w

# 多項式 f(x) が一次因子 x - a をもつときに商 f(x) / (x - a) を返す
def poldivlinfac(f,a):
        d = f[0]
        c = []
        k = 0
        for i in range(0,d):
                k = f[1][i] + a * k
                c.append(k)
        return [d-1,c]

# Newton 法を用いて f(x) = 0 の解をひとつ求める
def polnewton(f):
        seed()                  # 乱数の初期化
        e = pow(10,-12)         # 許容誤差
        g = polder(f)
        ok = 0
        while not ok:
                a = 0
                b = complex(random(),random())  # 初期値はランダムな複素数
                m = 0
                while m < 1024:                 # 最大 1024 回まで反復
                        a = b
                        b = a - poleval(f,a) / poleval(g,a)
                        m += 1
                        if abs(a-b) <= e:       # 収束していればOK
                                ok = 1
                                break
                        
        return b

# Newton 法を繰り返し用いて f(x) = 0 の解を全て求める
def polroots(f):
        r = []
        while f[0] > 0:
                a = polnewton(f)
                f = poldivlinfac(f,a)
                r.append(a)
                if 0:
                        print "[",
                        complexpolwrite([1,[1,-a]])
                        print "] * [",
                        complexpolwrite(f)
                        print "]"
        return r

# 実係数多項式を出力する
def polwrite(f):
        d = f[0]
        i = d
        while i >= 0:
                c = f[1][d-i]
                if c != 0:
                        if c > 0:
                                if i < d:
                                        print "+",
                        else:
                                print "-",
                        c = abs(c)
                        if c != 1 or i == 0:
                                print c,
                        if i > 1:
                                print "x^%d" % i,
                        if i == 1:
                                print "x",
                i -= 1

# 複素係数多項式を出力する
def complexpolwrite(f):
        d = f[0]
        i = d
        while i >= 0:
                c = f[1][d-i]
                if c != 0:
                        if c != 0:
                                if i < d:
                                        print "+",
                        print c,
                        if i > 1:
                                print "x^%d" % i,
                        if i == 1:
                                print "x",
                i -= 1

# 多項式を入力させる
def polinput():
        d = input(u"次数 = ")
        c = []
        i = d
        while i >= 0:
                print i,
                a = input(u"次の係数 = ")
                c.append(a)
                i -= 1
        return [d,c]

# 複素数の実部
def re(z):
        return z.real

# 複素数の虚部
def im(z):
        return z.imag

# メインルーチン
print u"代数方程式 f(x) = 0 の解を求めます。"
print u"f(x) を入力してください。"
f = polinput()
print
print u"f(x) =" ,
polwrite(f)
print
print
r = polroots(f)
print u"f(x) = 0 の解"
d = len(r)
for i in range(0,d):
        print r[i]

