#!/bin/env python
#-*- coding: utf-8 -*-
# 1行目は cygwin で使うときのおまじない
# 2行目は日本語コードの指定
#
# twinprime.py
# 双子素数の探索
# 2007.5.21-23

from math import *	# 数学関数を使うためのまじない
from random import *	# 乱数を使うためのまじない
from sys import *	# システム関数を使うためのまじない

# 素朴な素数判定法（n が小さい場合）
# n が素数なら 1, そうでなければ 0 を返す
def primetest(n):
	if n <= 1:
		return 0
	if n == 2:
		return 1
	if n % 2 == 0:
		return 0
	h = 1
	m = sqrt(n)
	i = 3L	
	while i <= m and h == 1:
		if n % i == 0:
			h = 0
		else:
			i = i + 2
	if h == 1:
		return 1
	else:
		return 0

# Jacobi 記号 : 後述の Solovay-Strassen 素数判定法で引用する
def jacobi(a, b):
	j = 1
	a = a % b
	if a < 0:
		if b & 3 == 3:
			j = -j
		a = -a
	while a > 1:
		while a & 1 == 0:
			b8 = b & 7
			if b8 == 3 or b8 == 5:
				j = -j
			a = a/2
		if a != 1:
			if a & 3 == 3 and b & 3 == 3:
				 j = -j
			t = b % a
			b = a
			a = t
	if a == 0:
		j = 0
	return j

#  自然数 n 未満の乱数ロング整数を返す関数
def rand(n):
	n0 = n
	x = 0L
	while n0 > 0:
		n0 = n0/10
		x = 10L * x + randint(0,9)	
		# randint(0,9) は 0 から 9 までの乱数整数
	return x % n

# 法 m での a*b
def modmul(a,b,m):
	return (a*b) % m

# 反復２乗法で a^e mod m を求める
# e >= 0 のとき限定
def powermod(a,e,m):
	x = 1
	b = a
	while e > 0:
		if e%2 == 1:
			x = modmul(x,b,m)
		b = modmul(b,b,m)
		e /= 2
	return x

# Solovay-Strassen 素数判定法
# n が素数なら 1、そうでなければ 0 を返す
def solovaystrassen(n):
	if n < 1048576:
		return primetest(n)
	else:
		if n % 2 == 0:
			return 0
		k = 0
		n0 = n
		while n0 > 0:
			k = k+1
			n0 = n0/32
		if k > 8:
			k = 8
		for i in range(0,k):
			a = rand(n)
			j = jacobi(a,n)
			e = powermod(a,(n-1)/2,n)
			if (j-e)%n != 0:
				return 0
		return 1

print u"双子素数を探索します。"
print
a = input(u"どこから探しますか（ 始めからなら 1 を入力 ）")
print

c = 0
k = (a + 1) / 6

if k <= 3:
	print u"1 組目の双子素数は 3 と 5",
	print
	c += 1

while 1:
        p = 6 * k - 1
        q = p + 2
        if solovaystrassen(p):
                if solovaystrassen(q):
                        c += 1
                        print u"%d 組目の双子素数は" % c,
                        print u"%d と %d" % (p,q)
                        if c % 20 == 0:
                                print
                                print u"( 続行は Enter, 終了は他のキー + Enter )"
                                ok = raw_input()
                                if len(ok) > 0:
                                        break
        k += 1

